// DUNG.H -- Dungeon class header file
#ifndef DUNG_H_
#define DUNG_H_

#include "ncounter.h"
#include "player.h"
//#include "ncounter.h"


const int XMAX = 78;
const int YMAX = 60;

const int RoomChance = 3;
const int HallChance = 7;

// Numbers are ASCII values for PrintMap
enum Features {NONE, EXC, TPT, STU, STD,
	ALT, FTN, STA, THN, ELV, PIT};
static const unsigned char FeatTypes[] =
	{' ', 232, '^', '<', '>', 236, 231, 234, 156, 136, 240};

istream& operator>> (istream& is, Features& F);

class Dungeon {
	bool D[XMAX][YMAX];
	bool Seen[XMAX][YMAX];
	bool Visited[XMAX][YMAX];
	bool Mapped[XMAX][YMAX];
	Features Feat[XMAX][YMAX];
	EncounterStack Encs [XMAX][YMAX];
	int Level;

	bool IsX (int x) {return x>=0 && x<XMAX ? true : false;}
	bool IsY (int y) {return y>=0 && y<YMAX ? true : false;}
	void HorizWall (int, int);
	void VertWall (int, int);
	void DrawBlock(int,int);
	unsigned char PrintChar (int, int);
	void SetSeen(int x, int y, bool val = true) {
		if (IsX(x) && IsY(y)) Seen[x][y] = val;
	}
	void SetVisited(int x, int y, bool val = true) {
		if (IsX(x) && IsY(y)) Visited[x][y] = val;
	}
	void SetMapped (int x, int y, bool val = true) {
		if (IsX(x) && IsY(y)) Mapped[x][y] = val;
	}
	bool IsVisited(int x, int y) {
		if (IsX(x) && IsY(y)) return Visited[x][y];
		return false;
	}
	bool IsMapped(int x, int y) {
		if (IsX(x) && IsY(y)) return Mapped[x][y];
		else return false;
	}
	void PrintFeat (int x, int y, int ScreenX, int ScreenY);
public:

	bool& Map (int x, int y) {
		static bool dummy = true;
		if (IsX(x) && IsY(y))
			return D[x][y];
		dummy = true;
		return dummy;
	}
	bool IsSolid (int x, int y) {
		if (IsX(x) && IsY(y) && D[x][y] != true) return false;
		else return true;
	}
	bool IsClear (int x, int y) { return !IsSolid (x,y);}
	Dungeon(int lev = 1);
	void PrintMap(int left = 0, int top = 0, int right = XMAX-1, int bottom = YMAX-1);
	void DrawView (int, int, bool Light);
	void RandSquare(int& x, int&y);
	Features& Feature(int x, int y) {
		static Features dummy = NONE;
		if (IsX(x) && IsY(y)) return Feat[x][y];
		dummy=NONE;
		return dummy;
	}
	int Play(Player& P, Features& Mech); // Mech is how the player reached the level
			// Return value is the new level, Mech will indicate how player left level
	bool ResolveFeat (Player& P, bool draw);
	void DrawMap(Player &P);
	void Evade(Player& P);
	void DoMapArea(int x, int y);
	void DoMapDung();
	void LargeMap(Player& P);

	void Save (ostream& os);
	void Load (istream& is);
};
#endif // DUNGEON_H_
