// dice.h -- Dice class by garth dighton
#ifndef DICE_H_GARTH
#define DICE_H_GARTH

#include <iostream.h>
#include <stdlib.h>
#include <time.h>

/* SeedRNG uses its constructor to initialize the random number generator.
   A global variable of this class is declared to ensure that the RNG is
   seeded.
*/
class SeedRNG {
public:
       SeedRNG() {
                    srandom( (int) time(0));
       }
};

// Creates one instance per compilation unit, but means I don't have to
// have a .cc file as well as a .h file. The RNG may get seeded several times,
// but this is no big deal.
static SeedRNG gdighton_seeding_RNG;

/* This is the main class of this library */
class Dice {
// This is the number of sides on the die.
      unsigned int sides;

protected:
      virtual int Roll () {
          return random() % sides + 1;
      }

public:
       // The constructor requires the number of sides on this particular
       // die.
       Dice (unsigned int S = 6) : sides (S) {}

       // The integer conversion operator rolls the die once, returning
       // the result.
       operator int () { return Roll(); }

       // The parenthesis operator accepts the number of dice to roll,
       // and returns the sum. The number defaults to 1.
       int operator() (int Num=1) {
	  	if (sides == 0) return 0;
		int sum(0);
          for (int i = 0; i<Num; i++ )
          	sum += Roll();
          return sum;
       }
	  virtual void ChangeType (unsigned int S) { sides = S;}
	  int Sides() {return sides;}
};

// An average die - equal chances for 2,3,3,4,4,5
class AverageDice {
	Dice d6;
	int Roll () {
	 	int x = d6;
		if (x == 1) x = 3;
		else if (x == 6) x = 4;
		return x;
	}
public:
	AverageDice () : d6(6) {}
	operator int() {return Roll();}
	int operator() (int Num=1) {
		int sum(0);
		for (int i=0; i<Num; i++)
			sum += Roll();
		return sum;
	}
};

// A random float between 0 and 1 (includes 1, not 0)
class FloatDice {
	Dice dMillion;
	double Roll () {
	 	return float(dMillion)/1000000.0;
	}
public:
	FloatDice() : dMillion (1000000) {}
	operator double() {return Roll();}
	double operator() (int Num = 1) {
		double sum(0);
		for (int i=0; i<Num; i++)
			sum += Roll();
		return sum;
	}
};

class FudgeDice {
	Dice d3;
	int Roll() {
		return d3-2;
	}
public:
	FudgeDice () : d3(3) {}
	operator int () {return Roll();}
	int operator () (int Num = 1) {
		int sum(0);
		for (int i=0; i<Num; i++)
			sum += Roll();
		return sum;
	}
};

// Multiplies dice rather than adding them; Default number is 2;
class MulDice {
 	Dice D;
public:
	MulDice (int sides) : D(sides) {}
	operator int () { return D * D; }
	int operator () (int Num = 2) {
		int product (1);
		for (int i=0; i<Num; i++)
			product *= D;
		return product;
	}
};

class OpenDice : public Dice
{
	int Threshold;
protected:
	int Roll () {
	 	int value = Dice::Roll();
		int result = value;
		while ((value = Dice::Roll()) > Threshold) result += value;
		return result;
	}
public:
	OpenDice (int S = 6) : Dice(S), Threshold(S-1) {}
	OpenDice (int S, int T) : Dice (S), Threshold (T) {}
	void ChangeType (int S) {
		Threshold = S-1;
		Dice::ChangeType(S);
	}
};

// Predefining some standard dice globally
static Dice			d4(4), d6(6), d8(8), d10(10), d12(12), d20(20), d100(100);
static AverageDice 	dAvg;
static FloatDice		dFloat;
static FudgeDice		dF;
static MulDice		d10x (10);
static OpenDice od8(8), od6(6), od4(4), od10(10), od12(12), od20(20,18), od100(100, 95);

#endif

