// PLAYER.H -- Header file for player class.
#ifndef PLAYER_H_
#define PLAYER_H_
#include "window.h"
#include "dice.h"
#include <iostream.h>
#include <math.h>

enum SpellType {NoSpell, MMissile, MinCure, MEvade, FireBall, MAgility,
	MDescend, MTeleport, MajCure, MAscend, MChangeLev, HealAll, LBolt,
	MapDung, MLight, MContLight, MapArea, MFortune, SupAgility,
	MStrength, MStoneSkin, VampDrain, MSpeedHeal, MTunnel
	};
// forward declaration:
class Monster;

class Player {
	// Window for displaying messages
	Window MessageWnd;
	// Window for displaying Player status
	Window StatusPn1, StatusPn2;
	inline int max (int a, int b) {return a>b ? a : b;}
	inline int min (int a, int b) {return a<b ? a : b;}

	inline int Use(int& Bonus) { return Bonus ? Bonus-- : 0;}
/*		if (Bonus) {
			Bonus -= 1; return Bonus+1;
		} else return 0;
	}*/
public:
	// Location in dungeon
	int x, y;
	int Level;
	char Name[20];
	// Basic stats
	int HitPoints, STR, DEX, CON, LUCK;
	int Experience;
	// Current Values
	int CurHP;
	// Temporary bonuses from magic;
	int Agility, Strength, Fortune, SpeedHeal, StoneSkin;
	int MagLight;
	// Bonuses from these items
	int Sword, Shield, Armor, Ring, Boots, Amulet;
	// Number of grenades and healing potions
	int Grenades, Potions;
	// PocketGold is gold from this adventure, BankGold is money from previous
	int PocketGold, BankGold;
	int TurnNo;	// Turn number;
	// Spell variables
	int Spells [4];
	int Wands [4];


	// Is automap turned on
	bool Map;
	Player(char N[] = "?????");

	bool SavingThrow (int L = 1) {
		int roll = d20 + max (LUCK-12, 0) + Level + Amulet + Use(Fortune);
		int roll2 = od20 + L + od10(L/2+1);
		if (roll > roll2) Experience += 5 * (roll - roll2);
		return (roll >= roll2 ? true : false);
	}

//	int AttackRoll() {return d8(Level+2) + Sword + max(DEX-12,0) + (Agility = max (Agility-1, 0));}
	int AttackRoll() {return d20 + Level + max(DEX-12, 0) + Sword +Use (Agility);}

//	int DefenseRoll() {return d8(Level+2) + Shield + max(DEX-12, 0) + Agility;}
	int DefenseRoll() {return d20 + Level + max(DEX-12, 0) + Shield + Agility;}

	int DamageRoll() {return od8(Level/3+1) + Sword + Level + max(STR-12,0) + Use(Strength);}

//	int EvadeRoll() {return d8(Level+2) + Boots + max (DEX-12, 0);}
	int EvadeRoll() {return d20 + Level + Boots + max(DEX-12, 0);}

	int TakeHit(int Damage) {
		Damage -= Armor;
		Damage -= Use(StoneSkin);
		Damage = max(Damage,1);
		CurHP -= Damage;
		return Damage;
	}

	int Heal (int Points) {
		int oldHP = CurHP;
		CurHP = min (CurHP + Points, HitPoints);
		return CurHP - oldHP;
	}

	void UpLevel ();

	void Turn();

	int XPNeeded() {return (int) pow(Level, 2.5) * 1000;}

	void MsgClear() {MessageWnd.Clear();}
	void Message (SString msg) {MessageWnd(msg);}
	void Status();

	void Save(ostream& os);
	void Load (istream& os);

	void SpellPrint();
	void SpellPrintCombat();
	bool LoseSpell(int L);
	SpellType SpellChoose ();
	SpellType SpellChoose (Monster& M);

	int Roll (int Num, int Sides);
	void Create();
	void Disenchant();
	void EnchantItem();
	void LoseAttribute();
	void GainAttribute();

	void Pause() {
		Message ("Press <Enter> to continue");
		while (getch() != 13);
	}
};

#endif	// PLAYER_H_
