// Window.h - encapsulates certain conio.h functions into a window class.

// Output in a window is done at the bottom of the window only, and is
// done in line-oriented fashion. The user must pass in an entire line
// of output, which is printed in the current color at the bottom of the
// window.

// The window is created with its borders set. They may not be changed,
// although a different window with other borders may be created and
// used. The color of the text may be changed, but you cannot have
// several colors on one line.

// When using this class it is best not to use any other form of output
// to the screen, as the results may be unpredictable. The Standard output
// streams ignore windowing, text coloring, and so forth. Using cin and
// scanf to get data from the user seems to be relatively safe, however,
// although the echoed output is printed in the standard color (LtGray,
// color #7


#ifndef WINDOW_H_
#define WINDOW_H_

#include <conio.h>
#include "sstring.h"
#include "stdio.h"

enum ConioColor {Black, Blue, Green, Cyan, Red, Purple, LtBrown, LtGray,
	DkGray, LtBlue, LtGreen, LtCyan, LtRed, Magenta, Yellow, White};

class Window {
	int left, top, right, bottom;
	ConioColor color;
	int row;
public:
	void Clear() {
		window (left, top, right, bottom);
		highvideo();
		clrscr();
		row=1;
	}
	Window (int L, int T, int R, int B) : left(L), top(T), right(R),
		bottom(B), color(LtGray) { Clear(); }
	void out (const SString& S, ConioColor C) {
		window (left, top, right, bottom);
		textcolor(C);

		if (row <= bottom-top+1) gotoxy(1,row);
		else gotoxy(1, bottom-top+1);
		if (row > bottom-top+1) cputs("\n\r");
		cputs(S);
		row++;
		fflush (stdout);
		textcolor (LtGray);
	}
	void out(const SString& S) {
		out (S, color);
	}
	void operator () (const SString& S) {
		out (S);
	}
	void operator() (const SString& S, ConioColor C) {
		out (S, C);
	}
	void Color(ConioColor C) {color = C;}
	ConioColor Color() {return color;}
};

#endif
