// ENCOUNTER.cc -- Implementation for encounter classes
#include "ncounter.h"
#include <math.h>
#include <ctype.h>

// A quick helper function;
int Encounter::Roll (int Number, int Sides)
{
	Dice D(Sides);
	return D(Number);
}

static int OpenRoll(int Number, int Sides, int Threshold) {
	OpenDice D(Sides, Threshold);
	return D(Number);
}

bool Monster::Resolve (Player& P)
{
	window (61,1,80,20);
	gotoxy(9,9);
	textcolor (LtGray);
	cprintf (MonsterSymbs[Type]);

	char buf[100];
	sprintf(buf, "There is a %s here", Name);
	P.Message (buf);

	bool VampiricDrain = false;

	do {
		P.Message ("(F)ight, (E)vade, (C)ast Spell, or Use (G)renade");
		char ch;
		do { ch = tolower(getch());
		} while (ch != 'f' && ch != 'e' && ch != 'c' && ch != 'g'
			&& ch != 13 && ch != '+' && ch != '0' && ch != '-');
		switch (ch) {
			case 'f':
			case 13:
				if (P.AttackRoll() < DefenseRoll()) {
					sprintf (buf, "You miss the %s.", Name);
					P.Message(buf);
				} else {
					int Damage = TakeHit(P.DamageRoll());
					sprintf (buf, "You do %d points of damage to the %s", Damage, Name);
					if (VampiricDrain) Heal(Damage);
					P.Message(buf);
				}
				break;
			case 'e':
			case '0':
				if (P.EvadeRoll() < PursuitRoll()) {
//				if (d10 < 3) {
					P.Message ("You cannot get away");
				} else {
					CurHP = HitPoints;
					window (61,1,80,20);
					gotoxy(9,9);
					textcolor (LtGray);
					cprintf ("   ");
					return true;
				}
				break;
			case 'c':
			case '+': {
				SpellType S = P.SpellChoose(*this);
				if (S == NoSpell) continue;
				if (S == MEvade) {
					CurHP = HitPoints;
					window (61,1,80,20);
					gotoxy (9,9);
					textcolor (LtGray);
					cprintf ("   ");
					return true;
				}
				if (S == VampDrain) VampiricDrain = true;
			}
				break;
			case 'g':
			case '-':
				if (P.Grenades) {
					P.Message ("You use a grenade and destroy the monster!");
					P.Grenades--;
					CurHP = 0;
				} else {
					P.Message ("You have no grenades to use!!");
					continue;
				}
				break;
		}
		if (CurHP > 0) {
			if (Type == Troll) Heal(int(sqrt(Level * Level*Level)));
			if (Type == Dragon && d10 == 1) {
				P.Message ("The Dragon breathes fire!!");
			 	int Damage = CurHP/2;
				if (P.SavingThrow(Level)) Damage /= 2;
				P.TakeHit (Damage);
				sprintf (buf, "You take %d points of damage", Damage);
				P.Message (buf);
			} else if (AttackRoll() < P.DefenseRoll()) {
			 	sprintf (buf, "The %s misses.", Name);
				P.Message (buf);
			} else {
				int Damage = P.TakeHit(DamageRoll());
				sprintf (buf, "The %s does %d points of damage", Name, Damage);
				P.Message (buf);
				if (Type == Vampire && !P.SavingThrow(Level)) P.LoseAttribute();
				if (Type == Balrog && d10 < 6 && !P.SavingThrow(Level))
					P.Disenchant();
				if ((Type == Ghoul || Type == Harpy) && d10 < 6
					&& !P.SavingThrow(Level))
				{
					if (Type == Ghoul) P.Message ("The Ghoul's touch paralyzes you");
					else P.Message ("You are mesmerized by the Harpy's song");
					if (AttackRoll() < P.DefenseRoll()) {
						sprintf (buf, "The %s misses.", Name);
						P.Message (buf);
					} else {
						int Damage = P.TakeHit(DamageRoll());
						sprintf (buf, "The %s does %d points of damage", Name,	Damage);
						P.Message (buf);
					}
				}
			}
		}
		P.Status();
		P.Message("------------------");
		if (P.CurHP <0) {
		 	sprintf (buf, "The %s has killed you", Name);
			P.Message (buf);
			P.Status();
			P.Pause();
			return false;
		}
	} while (CurHP >0);
	sprintf (buf, "The %s is dead", Name);
	P.Message(buf);
	P.Experience += Type * Level * 10 + HitPoints;
	P.Status();
	P.Pause();
	P.Turn();
	P.MsgClear();
	if (d10 == 1) {
		Encounter* Enc = new Treasure (Level);
		Enc->Resolve(P);
		delete Enc;
		if (P.CurHP < 0) return false;
	}
	window (61,1,80,20);
	gotoxy(9,9);
	textcolor (LtGray);
	cprintf ("   ");
	return false;
}


bool Treasure::Resolve(Player& P)
{
static char* TSymbs[] = {"$$$", "$$$", "$$$", "$$$", "***", "jwl", "swd", "arm",
	"shd", "rng", "scr", "cup", "tom", "hhg", "bts",
	"amt", "ptn", ":=:"};
	window (61,1,80,20);
	gotoxy(9,9);
	textcolor (LtGray);
	cprintf (TSymbs[Type]);

	char buf [100];
	sprintf (buf, "You find %s.", TNames[Type]);
	P.Message(buf);
	switch (Type) {
	 	case Chest:
			P.Message ("Open it? (Y/N) "); break;
		case Tome:
			P.Message ("Read it? (Y/N) "); break;
		case Cup:
			P.Message ("Drink it? (Y/N) "); break;
		default:
			P.Message ("Take it? (Y/N) "); break;
	}
	char ch;
	do {
		ch = getch();
	} while (ch != 'y' && ch != 'Y' && ch != 'n' && ch != 'N'
			&& ch != 13 && ch != '0');
	int value;
	if (ch == 'y' || ch == 'Y' || ch == 13) {
		if (d6 == 1 && Type != Cup) { // Cups of Health are never trapped
		 	// Treasure is Trapped
			if (P.SavingThrow(Level)) {
			 	P.Message ("You avoid a trap");
			} else {
			 	int dmg = Roll (Level, 3);
				sprintf (buf, "You take %d points of damage", dmg);
				P.Message (buf);
				P.CurHP -= dmg; // Don't use TakeHit() because Armor doesn't help
				if (P.CurHP <0) {
				 	P.Message ("You have Died.");
					P.Pause();
					return false;
				}
			}
		}

		switch (Type) {
			case Copper:
				value = (int) OpenRoll (Level, 10, 9);
				P.PocketGold += value;
				sprintf (buf, "You found %d Gold Pieces worth of copper", value);
				P.Message(buf);
				break;
			case Silver:
				value = (int) OpenRoll (Level, 50, 45);
				P.PocketGold += value;
				sprintf (buf, "You found %d Gold Pieces worth of silver", value);
				P.Message (buf);
				break;
			case Gold:
				value = (int) OpenRoll (Level, 100, 95);
				P.PocketGold += value;
				sprintf (buf, "You found %d Gold Pieces worth of gold", value);
				P.Message(buf);
				break;
			case Platinum:
				value = (int) OpenRoll (Level, 500, 475);
				P.PocketGold += value;
				sprintf (buf, "You found %d Gold Pieces worth of platinum", value);
				P.Message (buf);
				break;
			case Gems:
				value = (int) OpenRoll (Level, 1000, 950);
				P.PocketGold += value;
				sprintf (buf, "You found %d Gold Pieces worth of gems", value);
				P.Message (buf);
				break;
			case Jewel:
				value = (int) OpenRoll (Level, 2000, 1800);
				P.PocketGold += value;
				sprintf (buf, "It is worth %d Gold Pieces!", value);
				P.Message (buf);
				break;
			case Sword:
				value = Roll (1, Level);
				sprintf (buf, "The sword is a +%d sword!", value);
				P.Message(buf);
				if (value >= P.Sword) {
					P.Sword = value;
					P.Message ("You wield the sword");
				} else {
					P.Message ("You already have a better sword");
				}
				break;
			case Armor:
				value = Roll(1, Level);
				sprintf (buf, "The armor is +%d", value);
				P.Message (buf);
				if (value >= P.Armor) {
					P.Armor = value;
					P.Message ("You put on the magic armor");
				} else {
					P.Message ("You are already wearing better armor");
				}
				break;
			case Shield:
				value = Roll(1, Level);
				sprintf (buf, "The shield has a +%d bonus", value);
				P.Message (buf);
				if (value >= P.Shield) {
					P.Shield = value;
					P.Message ("You wear the new magical shield");
				} else {
					P.Message ("You already have a better shield");
				}
				break;
			case Ring:
				value = Roll(1, Level);
				sprintf (buf, "The ring heals %d Hit Points per turn", value);
				P.Message (buf);
				if (value >= P.Ring) {
					P.Ring = value;
					P.Message ("You put on the ring");
				} else {
					P.Message ("Your current ring will help you more");
				}
				break;
			case Wand:
				value = d4;
				P.Wands[value-1] ++;
//				P.Wands[0] += Roll(1, 2*Level);
//				P.Wands[1] += Roll(1, Level);
//				P.Wands[2] += Roll(1, Level/2);
//				P.Wands[3] += Roll(1, Level/4);
//				P.Message("The wand stores additional spell energy");
				sprintf (buf, "The scroll contains a spell of Level %d", value);
				P.Message(buf);
				break;

			case Cup:
				if (P.CurHP < P.HitPoints) {
					P.Message ("You feel better");
					P.Heal (Roll(Level, 6));
				} else {
					P.Message ("You feel much healthier");
					P.CurHP = ++P.HitPoints;
				}
				break;
			case Tome:
				TomeEffects(P);
				if (P.CurHP <0) {
				 	P.Message ("You have died!!");
					P.Pause();
					return false;
				}
				break;
			case Grenade:
				P.Message ("You take the Holy Hand Grenade");
				P.Grenades ++;
				break;
			case ElfBoots:
				value = Roll(1, Level);
				sprintf (buf, "The boots grant +%d to speed", value);
				P.Message(buf);
				if (value >= P.Boots) {
					P.Boots = value;
					P.Message ("You put on the Elven Boots");
				} else {
					P.Message ("Your current footgear is much superior");
				}
				break;
			case Amulet:
				value = Roll(1, Level);
				sprintf (buf, "The amulet grants a +%d bonus to saving throws", value);
				P.Message(buf);
				if (value >= P.Amulet) {
					P.Amulet = value;
					P.Message ("You wear the Magic Amulet");
				} else {
					P.Message ("Your current amulet will assist you more");
				}
				break;
			case Potion:
				P.Potions++;
				P.Message ("You take the healing potion");
				break;
			case Chest:
				value = od4;	// Opensided d4
				P.Message ("You open the chest...");
		  		P.Message ("---------------------");
				for (int i=1; i<= value; i++) {
					Treasure* Enc = NULL;
					do {
						if (Enc) delete Enc;
						Enc = new Treasure(Level);
					} while (Enc->Type == Chest);
					Enc->Resolve(P);
					delete Enc;
					if (P.CurHP <0) return false;
					P.Turn();

				}
				P.Message ("There is nothing else in the chest");
		}
		P.Status();
	 	P.Pause();
	}
	P.Message ("---------------------------");

	window (61,1,80,20);
	gotoxy(9,9);
	textcolor (LtGray);
	cprintf ("   ");
	P.Turn();
	return false;
}

Monster::Monster (int L = 1)
	: Encounter(L), Type(MonsterType (Roll (1, Dragon+1)-1)), DamageDie(Type+4)
{

	CurHP = HitPoints = DamageDie(Level*Level);
	if (Level%10 == 1)
		sprintf(Name, "%dst Level %s", Level, MonsterNames[Type]);
	else if (Level%10 == 2)
		sprintf(Name, "%dnd Level %s", Level, MonsterNames[Type]);
	else if (Level%10 == 3)
		sprintf(Name, "%drd Level %s", Level, MonsterNames[Type]);
	else sprintf(Name, "%dth Level %s", Level, MonsterNames[Type]);
}

Treasure::Treasure (int L = 1) : Encounter(L)
{
	static int Table [] = {15, 12, 9, 6, 3, 1, 3, 3, 3, 3, 3, 15,
		9, 3, 3, 3, 3, 10};
	int roll = Roll(1, 107);
	int i = 0; int sum=0;
	while ((sum += Table[i]) < roll) i++;
	Type = TreasureType (i);
}

void Treasure::TomeEffects(Player& P)
{
	if (d10 > 2) {		// Good Effects
		int value = Roll(1, 5);
		switch (value) {
			case 1:
				P.GainAttribute();
				break;
			case 2: P.Experience += d100(Level);
				P.Message ("You are more powerful");
				break;
			case 3: {
				P.Message ("A Treasure appears");
				Encounter* Enc = new Treasure (Level);
				Enc->Resolve(P);
			 	delete Enc;
				ungetch(13);		// Avoid double returns
				if (P.CurHP < 0) return;
				break;
			}
			case 4:
				P.EnchantItem();
				break;
			case 5:
				P.Message ("You are completely healed");
				P.CurHP = P.HitPoints;
				P.Spells[0] = P.Level;
				P.Spells[1] = P.Level/2;
				P.Spells[2] = P.Level/4;
				P.Spells[3] = P.Level/8;
				break;
		}
	} else {		// Bad effects
		int value = Roll(1, 4);
		switch (value) {
		 	case 1:
				P.Message ("Some of your money dissapears!!");
				P.PocketGold /= 2;
				break;
			case 2:
				P.LoseAttribute();
				break;
			case 3:
				P.Disenchant();
				break;
			case 4:
				P.Message ("A Monster appears");
				Encounter* Enc = new Monster (Level);
				while (Enc->Resolve(P)) {
					P.Message ("A mysterious force prevents you from escaping");
					P.Pause();
				}
			 	delete Enc;
				ungetch(13);		// Avoid double returns
				if (P.CurHP < 0) return;
				break;
		}

	}
}


void Treasure::Save(ostream& os) {
 	os << "2 " << Level << ' ' << Type << endl;
}

void Monster::Save(ostream& os) {
	os << "1 " << Level << ' ' << Type << ' ' << CurHP << ' ' << HitPoints;
	os << ' ' << DamageDie.Sides() << endl;
	os << Name << endl;
}

Treasure::Treasure (istream& is) {
	is >> Level >> Type;
}

Monster::Monster (istream& is) {
 	is >> Level >> Type >> CurHP >> HitPoints;
	int S;
	is >> S; DamageDie.ChangeType (S);
	is.getline(Name, 20);
}

istream& operator>> (istream& is, TreasureType& T) {
 	int I;
	is >> I;
	T = (TreasureType) I;
	return is;
}
istream& operator>> (istream& is, MonsterType& M) {
 	int I;
	is >> I;
	M = (MonsterType) I;
	return is;
}
