// Encounter.h -- Encounter classes

#ifndef ENCOUNTER_H_
#define ENCOUNTER_H_

#include "player.h"
//#include "dung.h"
#include <stdio.h>
#include <math.h>

class Encounter {
protected:
	int Level;

	int Roll (int Number, int Sides);
public:
	Encounter (int L = 1) {
		if (d10 < 7) Level = L;
		else	Level = int(float(od100(10)) / 1000.0 * 2.0 * (float) L) +1;
	}
	virtual bool Resolve(Player& P) = 0;
	virtual void Save (ostream& os) = 0;
	virtual bool IsTreasure() = 0;
	virtual bool IsMonster() = 0;
};

class EncounterStack {
	struct link {
		Encounter* Data;
		link* next;
		link(Encounter* D) : Data(D), next(NULL) {}
	} *head;
public:
	EncounterStack() : head(NULL) {}
	void Push (Encounter* D) {
		link* L = new link (D);
		L->next = head;
		head = L;
	}

	Encounter* Pop () {
		if (head) {
     		link* L = head;
     		head = head->next;
     		Encounter* D = L->Data;
     		delete L;
     		return D;
		} else return NULL;
	}
	operator bool() { return (bool) head;}

	~EncounterStack() {
		link* temp = head;
		while (head) {delete temp; temp = head; head = head->next;}
	}
	bool IsTreasure() {
		if (head) return head->Data->IsTreasure();
		else return false;
	}
	bool IsMonster() {
		if (head) return head->Data->IsMonster();
		else return false;
	}

};

static char* TNames[] = { "a pile of copper", "a pile of silver",
	"a pile of gold", "a pile of platinum", "a pile of gems",
	"a jewel", "a magic sword", "magical armor", "a magic shield",
	"a magical ring", "a magic scroll",
	"a Cup of Health", "a mighty tome", "a Holy Hand Grenade",
	"Elven Boots", "an Amulet", "a healing potion", "a chest"
};

enum TreasureType {Copper, Silver, Gold, Platinum, Gems, Jewel, Sword,
	Armor, Shield, Ring, Wand, Cup, Tome,
	Grenade, ElfBoots, Amulet, Potion, Chest};
istream& operator>> (istream& is, TreasureType& T);

class Treasure : public Encounter {
	TreasureType Type;

	void TomeEffects(Player& P);
public:
	Treasure (int L = 1);
	Treasure (istream& is);

	bool Resolve (Player& P);
	void Save (ostream& os);
	bool IsTreasure() {return true;}
	bool IsMonster () {return false;}
};

enum MonsterType {Kobold, Goblin, Harpy, Orc, Ghoul, Ogre, Troll, Vampire,
		Giant, Balrog, Dragon};
istream& operator>> (istream& is, MonsterType& M);

static const char* MonsterNames[] = {"Kobold", "Goblin", "Harpy",
	"Orc", "Ghoul", "Ogre", "Troll", "Vampire", "Giant", "Balrog", "Dragon"};
static const char* MonsterSymbs[] = {"KBD", "GOB", "HPY", "ORC", "GHL",
	"OGR", "TRL", "VMP", "GNT", "BLG", "DGN"};

class Monster: public Encounter {
	MonsterType Type;
	int CurHP, HitPoints;
	// Does double duty, serving as both the amount of damage the monster does
	// and the hit die for determining hit points, also figures in other rolls
	OpenDice DamageDie;
	int max(int a, int b) {return a> b ? a : b;}

	int AttackRoll() {return od20 + Level *3;}
	int DefenseRoll() {return od20 + Level *3;}
	int DamageRoll() {return DamageDie(Level/3+1) + (int) sqrt(Level*Level*Level) - 1;}
	int PursuitRoll() {return od20 + Level*2;}


//	bool SavingThrow(int L=1) {return d20 < LUCK - L ? true : false;}

public:
	char Name[20];
	int TakeHit(int Damage) {
		CurHP -= Damage;
		return Damage;
	}
	int Heal (int Points) {
		int oldHP = CurHP;
		CurHP += Points;
		if (CurHP > HitPoints) CurHP = HitPoints;
		return CurHP - oldHP;
	}

	Monster (int L=1);
	Monster (istream& is);
	bool Resolve (Player& P);
	void Save (ostream& os);
	bool IsTreasure () {return false;}
	bool IsMonster () {return true;}
};


#endif //ENCOUNTER_H_
